<?php

namespace Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;
use \Teknisa\GER\Services\CrudService;
use \Teknisa\GER\Utilities\Messages;
// use Teknisa\COC\Controllers;
use Zeedhi\Framework\ORM\DateTime;


class AlteracoesReceita extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = '';
    protected $newCode;
    protected $modpreprecmTabela;
    protected $environment;
    protected $parameterBag;
    protected $connection;


    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $modpreprecmTabela, $newCode, WorkflowReceitaController $WorkflowReceitaController, $connection) {
        $this->dataSourceManager = $dataSourceManager;
        $this->environment=$environment;
        $this->modpreprecmTabela = $modpreprecmTabela;
        $this->newCode=$newCode;
        $this->WorkflowReceitaController = $WorkflowReceitaController;
        $this->connection = $connection;
        parent::__construct($crudService, $parameterBag);
    }
    
    private function getRowsFromRequest($request){
        $rows = array();
        if ($request instanceof DTO\Request\Row) {
            $rows = array($request->getRow());
        }
        if ($request instanceof DTO\Request\DataSet) {
            $rows = $request->getDataSet()->getRows();
        }
        return $rows;
    }
    
    public function saveProduto(DTO\Request $request, DTO\Response $response){
        $this->aplicaEVerificaAlteracoes('proreceitam', $request, $response, 'save', 'default');
    }
    public function saveSubReceita(DTO\Request $request, DTO\Response $response){
        $this->aplicaEVerificaAlteracoes('subreceitam', $request, $response, 'save', 'default');
    }
    public function deleteSubReceita(DTO\Request $request, DTO\Response $response){
        $this->aplicaEVerificaAlteracoes('subreceitam', $request, $response, 'delete', 'default');
    }
    public function savePreparo(DTO\Request $request, DTO\Response $response){
        $this->aplicaEVerificaAlteracoes('modpreprecm', $request, $response, 'save', 'default');
    }
    public function deletePreparo(DTO\Request $request, DTO\Response $response){
        $this->aplicaEVerificaAlteracoes('modpreprecm', $request, $response, 'delete', 'default');
    }
    public function saveTesteReceita(DTO\Request $request, DTO\Response $response){
        $this->aplicaEVerificaAlteracoes('testereceitam_operador', $request, $response, 'save', 'default');
    }
    public function deleteTesteReceita(DTO\Request $request, DTO\Response $response){
        $this->aplicaEVerificaAlteracoes('testereceitam_operador', $request, $response, 'delete', 'default');
    }
    public function saveUnidade(DTO\Request $request, DTO\Response $response){
        $this->aplicaEVerificaAlteracoes('filireceita', $request, $response, 'save', 'default');
    }
    public function saveClassificacao(DTO\Request $request, DTO\Response $response){
        $this->aplicaEVerificaAlteracoes('categoriarec', $request, $response, 'save', 'classif');
    }
    public function deleteClassificacao(DTO\Request $request, DTO\Response $response){
        $this->aplicaEVerificaAlteracoes('categoriarec', $request, $response, 'delete', 'classif');
    }
    
    private function aplicaEVerificaAlteracoes($dataSourceName, DTO\Request $request, DTO\Response $response, $saveMethod, $verifMethod){
        $this->dataSourceName = $dataSourceName;
        try{
            $this->connection->beginTransaction();
            
            if ($saveMethod == 'save') {
                $this->saveCrudAlteracoes($request, $response);
            } else if ($saveMethod == 'delete') {
                $this->deleteCrudAlteracoes($request, $response);
            }
        
            $rows = $this->getRowsFromRequest($request);
            
            if ($verifMethod == 'classif') {
                $this->verificaAlteracoesClassif($request,$response,$rows);
            } else {
                $this->verificaAlteracoes($request,$response,$rows);
            }
            
            $this->connection->commit();
        } catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollback();
        }
    }
    
    private function saveCrudAlteracoes(DTO\Request $request, DTO\Response $response) {
		$dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
		$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
		$this->crudService->save($dataSourceName, $rows);
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
	}
	
	private function deleteCrudAlteracoes(DTO\Request $request, DTO\Response $response) {
		$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $dataSet = $request->getDataSet();
        $dataSet->setDataSourceName($this->dataSourceName);
        $this->crudService->delete($response, $dataSet);
	}

    public function verificaAlteracoesClassif(DTO\Request $request, DTO\Response $response, $rows){
            
        foreach($rows as $row) { 

            $pkCDRECEITA = $row['CDRECEITA'];
            
            $rowReceita = array(
                'CDRECEITA' => $pkCDRECEITA,
                '__is_new' => false
            );
            $this->salvaAlteracoes($request,$response, $rowReceita, 'receitam');
        }
    }
    
    public function verificaAlteracoes(DTO\Request $request, DTO\Response $response, $rows){
        
        foreach($rows as $row) { 

            $pkCDRECEITA = $row['CDRECEITA'];
  
            $pkNRVARIACAOREC = $row['NRVARIACAOREC'];
            
            $rowReceita = array(
                'CDRECEITA' => $pkCDRECEITA,
                '__is_new' => false
            );
            $this->salvaAlteracoes($request,$response, $rowReceita, 'receitam');
            $rowVariacao = array(
                'CDRECEITA' => $pkCDRECEITA,
                'NRVARIACAOREC' => $pkNRVARIACAOREC,
                '__is_new' => false
            );
            $this->salvaAlteracoes($request,$response, $rowVariacao, 'variacaorec');
        }
    }
    
    private function salvaAlteracoes(DTO\Request $request, DTO\Response $response, $row, $dataSourceName){
        
        $dataAtual = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $operador = $this->environment->getCdOperador();
        $row['DTULTATU'] = $dataAtual;
        $row['DSOPERULTATU'] = $operador;
    
        $method = $request->getMethod();
        $routePath = $request->getRoutePath();
        $userId = $request->getUserId();
        
        $newRequest = new DTO\Request\Row($row, $method, $routePath, $userId);
        $rows = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
		$this->crudService->save($dataSourceName, $rows);
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    }
    
}