<?php

namespace Controllers;

use Teknisa\GER\Utilities\Environment;
use Teknisa\GER\Services\CrudService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Remote\FileUploader\Server;

class CompanhiaController extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName = "companhia_cliente_operador";
	protected $dataSourceCompanhia = "companhia";
	protected $dataSourceCliente = "cliente";
	protected $dataSourceNameToSave;
	protected $novoCodigo;
	protected $environment;
	protected $fileUploader;
	protected $companhiaService;
	protected $cdcliente;
	protected $connection;
	protected $pathKey;
	protected $fileserverPath;
	protected $undefinedImagePath;

	public function __construct(CrudService $crudService, ParameterBag $parameterBag, Server $fileUploader, $fileServerParams, $companhiaService, $environment, $novoCodigo, $connection) {
		$this->fileUploader = $fileUploader;
		$this->companhiaService = $companhiaService;
		$this->novoCodigo = $novoCodigo;
		$this->environment = $environment;
		$this->fileserverPath = $fileServerParams->getFileserverPath();
		$this->pathKey = $fileServerParams->getPathKey();
		$this->undefinedImagePath = $fileServerParams->getUndefinedImagePath();
		$this->connection = $connection;

		parent::__construct($crudService, $parameterBag);
	}

	public function save(DTO\Request $request, DTO\Response $response) {
		try {
			$this->connection->beginTransaction();
			//Scope transaction
				$this->salvaCliente($request, $response);
				$this->salvaCompanhia($request, $response);
			//Commit
			$this->connection->commit();
		} catch (\Exception $e) {
			//Rollback
			$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}

	}

	public function delete(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$oldFglogocia = $this->companhiaService->getOldLogocia($row['CDCIA'], $row['NRORG']);
		if (($oldFglogocia) && ($oldFglogocia != $this->undefinedImagePath)){
			$this->fileUploader->deleteFile($oldFglogocia);
		}
		parent::delete($request, $response);
	}

	public function find(DTO\Request\Filter $request, DTO\Response $response) {
		$response2 = new DTO\Response();
		parent::find($request, $response2);
		if ($response2->getError()) {
			$response->setError($response2->getError());
		} else {
			$rows = array();
			foreach($response2->getDataSets()[0]->getRows() as $row) {
				$row['PHOTO_UPLOAD'] = [];
				$row['FGLOGOCIA'] = $row['FGLOGOCIA'] ? $this->fileserverPath.$row['FGLOGOCIA'] : null;
				$rows[] = $row;
			}
			$response->addDataSet(new DataSet($this->dataSourceName, $rows));
		}
	}

	public function salvaCliente(DTO\Request $request, DTO\Response $response){
		$this->dataSourceNameToSave = $this->dataSourceCliente;
		$row = $request->getRow();
		$inscricaoLimpa = $this->limpaInscricao($row['NRINSJURCLIE']);
		$tipoinscricaoJuridica = $this->tipoInscricao($inscricaoLimpa);
		if($row['__is_new']){
			$this->cdcliente = $this->preparaCodigoCliente($row['NRINSJURCLIE'], $tipoinscricaoJuridica);
			$novaRow = array(
				'CDCLIENTE'     => $this->cdcliente,
				'IDTPIJURCLIE'  => $tipoinscricaoJuridica,
				'NRINSJURCLIE'  => $inscricaoLimpa,
				'NRINSESTCLIE'  => $row['NRINSESTCLIE'],
				'NMRAZSOCCLIE'  => $row['NMRSOCCIA'],
				'NMFANTCLIE'    => $row['NMCIA']
			);
		} else {
			$this->cdcliente = $row['CDCLIENTE'];
			$novaRow = array(
				'CDCLIENTE'     => $row['CDCLIENTE'],
				'IDTPIJURCLIE'  => $this->tipoInscricao($row['NRINSJURCLIE']),
				'NRINSJURCLIE'  => $inscricaoLimpa,
				'NRINSESTCLIE'  => $row['NRINSESTCLIE'],
				'NMRAZSOCCLIE'  => $row['NMRSOCCIA'],
				'NMFANTCLIE'    => $row['NMCIA']
			);
		}
		$novaRow['changed'] = true;
		$novaRow['__is_new'] = $row['__is_new'];

		$rows = array();
		array_push($rows, $novaRow);

		$dataSet = new DataSet($this->dataSourceCliente, $rows);
    $newRequest = new DTO\Request\DataSet($dataSet, '','','');
    $newRequest = $this->crudService->buildDataSetIfRow($newRequest, $this->dataSourceCliente);
    parent::save($newRequest, $response);

	}

	private function limpaInscricao($nrinsjurclie){
		$caracteresEspeciais = array('.', '/', '-');
		return str_replace($caracteresEspeciais, '', $nrinsjurclie);
	}

	private function preparaCodigoCliente($nrinsjurclie, $tipoinscricaojuridica){
		$inscricaoLimpa = $this->limpaInscricao($nrinsjurclie);
		return $tipoinscricaojuridica.$inscricaoLimpa;
	}

	private function tipoInscricao($nrinscricao){
		$tamanhoString = strlen($nrinscricao);
		if ($tamanhoString == 11) {
      return 'F';
    } elseif ($tamanhoString == 14) {
      return 'J';
    } elseif ($tamanhoString <= 5) {
      return 'O';
    } else {
      return 'I';
    }
	}

	public function salvaCompanhia(DTO\Request $request, DTO\Response $response){

		$this->dataSourceNameToSave = $this->dataSourceCompanhia;
		$row = (array)$request->getRow();
		$row['CDCLIENTE'] = $this->cdcliente;

		if($row['__is_new']){
			$row['CDCIA'] = $this->novoCodigo->getCodeByTableName(strtoupper($this->dataSourceCompanhia), $this->environment->getNrOrgTrab(), 1, 4);
		}

		if (isset($row['PHOTO_UPLOAD'][0])){
			//REMOVE O ANTIGO ARQUIVO DO SERVIDOR
			if (!$row['__is_new']){
				$oldFglogocia = $this->companhiaService->getOldLogocia($row['CDCIA'], $row['NRORG']);
				if (($oldFglogocia) && ($oldFglogocia != $this->undefinedImagePath)){
					$this->fileUploader->deleteFile($oldFglogocia);
				}
			}
			//SALVA O NOVO PATH NO BANCO
			$row['FGLOGOCIA'] = $row['PHOTO_UPLOAD'][0]['path'];
		} else if ($row['__is_new']) {
			$row['FGLOGOCIA'] = null;
		} else {
			$row['FGLOGOCIA'] = str_replace($this->fileserverPath, "", $row['FGLOGOCIA']);
		}
		$rowsDataset = array($row);
		$dataSetNovaCompanhia = new DataSet("novacompanhia", $rowsDataset);
		$newRequest = new DTO\Request\DataSet($dataSetNovaCompanhia, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
		parent::save($newRequest, $response);
		$response->addDataSet($dataSetNovaCompanhia);
	}

	public function validateDuplicationSigla(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
        $conditions = [
            array(
                "columnName" => "SGCIA",
                "operator"   => 'LIKE_I',
                "value"      => $row['SGCIA']
            ),
            array(
                "columnName" => "NRORG",
                "operator"   => '=',
                "value"      => $this->environment->getNrOrgTrab()
            )
        ];
        $dataSet =  $this->crudService->getFilteredDataSet($this->dataSourceCompanhia, $conditions);
        if(current($dataSet->getRows())) {

            $errorMsg = "Sigla da Companhia ja cadastrada!";
            return $response->setError(new DTO\Response\Error($errorMsg, 1));
        }
	}
	
	public function saveClientAddress(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		// $idClient = $row['IDCLIENT'];
		$stateName = isset($row['NMESTADO']) ? htmlspecialchars($row['NMESTADO']) : null;
		$stateAbbrv = isset($row['SGESTADO']) ? htmlspecialchars($row['SGESTADO']) : null;
		$cityName = isset($row['NMMUNICIPIO']) ? htmlspecialchars($row['NMMUNICIPIO']) : null;
		$cityCode = isset($row['CDMUNICIPIO']) ? htmlspecialchars($row['CDMUNICIPIO']) : null;
		$idtpEndeClie = isset($row['IDTPENDECLIE']) ? htmlspecialchars($row['IDTPENDECLIE']) : null;
		$idtpEndeClie_edit = isset($row['IDTPENDECLIE_EDIT']) ? htmlspecialchars($row['IDTPENDECLIE_EDIT']) : null;
		$address = isset($row['DSENDECLIE']) ? htmlspecialchars($row['DSENDECLIE']) : null;
		$complement = isset($row['DSCOMPENDCLI']) ? htmlspecialchars($row['DSCOMPENDCLI']) : null;
		$countryCode = isset($row['CDPAIS']) ? htmlspecialchars($row['CDPAIS']) : null;
		$countryName = isset($row['NMPAIS']) ? htmlspecialchars($row['NMPAIS']) : null;
		$neighborhood = isset($row['NMBAIRCLIE']) ? htmlspecialchars($row['NMBAIRCLIE']) : null;
		$zipCode = isset($row['NRCEPCLIE']) ? htmlspecialchars($row['NRCEPCLIE']) : null;
		$telephone = isset($row['NRTELECLIE']) ? htmlspecialchars($row['NRTELECLIE']) : null;
		$ramal = isset($row['NRRAMALCLIE']) ? htmlspecialchars($row['NRRAMALCLIE']) : null;
		$email = isset($row['DSEMAILCLIECOP']) ? htmlspecialchars($row['DSEMAILCLIECOP']) : null;
		$nrorg = isset($row['NRORG']) ? htmlspecialchars($row['NRORG']) : null;
		$nrTelexClie = isset($row['NRTELEXCLIE']) ? htmlspecialchars($row['NRTELEXCLIE']) : null;
		$nrFaxClie = isset($row['NRFAXCLIE']) ? htmlspecialchars($row['NRFAXCLIE']) : null;
		$clientCode = isset($row['CDCLIENTE']) ? htmlspecialchars($row['CDCLIENTE']) : null;
		$isNew = $row['__is_new'];
		try {
			if($isNew) {
				$this->companhiaService->saveClientAddress(
					$stateAbbrv,
					$cityCode,
					$idtpEndeClie,
					$address,
					$complement,
					$countryCode,
					$neighborhood,
					$zipCode,
					$telephone,
					$ramal,
					$email,
					$nrorg,
					$nrTelexClie,
					$nrFaxClie,
					$clientCode);
			} else {
				$this->companhiaService->editClientAddress(
					$stateAbbrv,
					$cityCode,
					$idtpEndeClie,
					$address,
					$complement,
					$countryCode,
					$neighborhood,
					$zipCode,
					$telephone,
					$ramal,
					$email,
					$nrorg,
					$nrTelexClie,
					$nrFaxClie,
					$clientCode);
			}
		} catch (\Exception $exception) {
			throw $exception;
		}
	}

}