<?php

namespace Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;
use \Teknisa\GER\Services\CrudService;
use Utilities\Messages;
use Controllers;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class PrecoseciamController extends \Teknisa\GER\Controllers\TeknisaCrud{

	const NAO_POSSUI_PLANO_DE_VOO = 0;
	const PROPORCAO_HUM_POR_PAX = "1";
	const PROPORCAO_VARIAVEL = "2";
	const PROPORCAO_TABELA = "3";
	const PROPORCAO_LOADING = "4";
	const PROPORCAO_QUANTIDADE_FIXA = "5";

	protected $dataSourceName = 'precoseciam';
	protected $dataSourceNameTabprecose = 'tabprecosem';
	protected $dataSourceNamePrecoPromo = 'proprecosem';
	protected $dataSourceManager;
	protected $parameterBagColumns = array();
	protected $messages;
	protected $tabprecoseController;
	protected $newCode;
	protected $connection;

	function __construct(CrudService $crudService, ParameterBag $parameterBag, $dataSourceManager, Messages $messages, TabprecoseController $tabprecoseController, $workflowService, $newCode, $connection, $MessagesProcedure, $importarItensService) {
		$this->dataSourceManager = $dataSourceManager;
		$this->messages = $messages;
		$this->tabprecoseController = $tabprecoseController;
		$this->workflowService = $workflowService;
		$this->newCode = $newCode;
		$this->connection = $connection;
        $this->MessagesProcedure = $MessagesProcedure;
        $this->importarItensService = $importarItensService;
		parent::__construct($crudService, $parameterBag);
	}
	
	private function retornaQuantidadeSubGrupos($cdfilial, $cdcia, $cdmancia, $cdclasse, $cdciclo, $nrservicociafili) {
		$sql = "SELECT COUNT(1) AS QUANTIDADE_SUBGRUPOS
	       			FROM PRECOSESUBGRU P
	            		INNER JOIN SUBGRUSEMAM S ON S.CDFILIAL = P.CDFILIAL AND
	                                        S.CDCIA = P.CDCIA AND
	                                        S.CDMANCIA = P.CDMANCIA AND
	                                        S.NRSERVICOCIAFILI = P.NRSERVICOCIAFILI AND
	                                        S.CDCLASSE = P.CDCLASSE AND
	                                        S.CDSUBGRU = P.CDSUBGRU AND
	                                        S.NRSEQSUBGRUSEMAM = P.NRSEQSUBGRUSEMAM
				    WHERE P.CDFILIAL = :CDFILIAL
				        AND P.CDCIA = :CDCIA
				        AND P.CDMANCIA = :CDMANCIA
				        AND P.CDCLASSE = :CDCLASSE
				        AND P.CDCICLO = :CDCICLO
				        AND P.NRSERVICOCIAFILIPRECO = :NRSERVICOCIAFILI";

		$stmt = $this->connection->prepare($sql);
		$stmt->execute(array(
			"CDFILIAL" => $cdfilial,
			"CDCIA" => $cdcia,
			"CDMANCIA" => $cdmancia,
			"CDCLASSE" => $cdclasse,
			"CDCICLO" => $cdciclo,
			"NRSERVICOCIAFILI" => $nrservicociafili
		));
		return $stmt->fetch(\PDO::FETCH_ASSOC);
	}

	private function retornaQuantidadeManuais($cdfilial, $cdcia, $nrtabpreco, $nrservicociafili, $cdclasse, $cdciclo){
		$sql = "SELECT MAX(CDMANCIA), COUNT(1)
             		FROM (SELECT DISTINCT P.CDMANCIA 
	                     FROM PRECOSESUBGRU P 
	                               INNER JOIN TABPRECOMENUGRID T ON T.CDFILIAL = P.CDFILIAL AND T.CDCIA = P.CDCIA AND T.CDMANCIA = P.CDMANCIA
	                    WHERE T.CDFILIAL = :CDFILIAL
	                      AND T.CDCIA = :CDCIA
	                      AND T.NRTABPRECO = :NRTABPRECO 
	                      AND P.CDFILIAL = :CDFILIAL
	                      AND P.CDCIA = :CDCIA
	                      AND P.NRSERVICOCIAFILIPRECO = :NRSERVICOCIAFILI
	                      AND P.CDCLASSE = :CDCLASSE
	                      AND P.CDCICLO = :CDCICLO)";
		$stmt = $this->connection->prepare($sql);
		$stmt->execute(array(
			"CDFILIAL" => $cdfilial,
			"CDCIA" => $cdcia,
			"NRTABPRECO" => $nrtabpreco,
			"NRSERVICOCIAFILI" => $nrservicociafili,
			"CDCLASSE" => $cdclasse,
			"CDCICLO" => $cdciclo
		));
		return $stmt->fetch(\PDO::FETCH_ASSOC);
	}

	private function retornaProporcoes($cdfilial, $cdcia, $cdmancia, $cdclasse, $cdciclo, $nrservicociafili, $dtinivigse, $dtfimvigse) {
// 		var_dump($cdfilial,
// $cdcia,
// $cdmancia,
// $cdclasse,
// $cdciclo,
// $nrservicociafili,
// $dtinivigse,
// $dtfimvigse
// );die;		
		$valorQtproporcaoDiferente = 'N';
		$valorQtproporcao          = null;
		$valorQuantidade           = 0;
		$idpropsubsemam            = null;
		
		$sql = "SELECT S.CDSUBGRU, 
		               S.NRSEQSUBGRUSEMAM, 
		               S.IDPROPSUBSEMAM,
                       S.DTINIVGSUMA,
                       S.DTFIMVGSUMA,
				       P.NRSERVICOCIAFILIPRECO,
				       P.NRSERVICOCIAFILI,
				       P.CDPRODUTO,
				       P.CDRECEITA,
		               CASE 
		                 WHEN S.IDPROPSUBSEMAM IN ('1', '3') 
		                   THEN NULL 
		                 ELSE S.QTPROPSUBSEMAM 
		               END AS QTPROPSUBSEMAM
                  FROM PRECOSESUBGRU P
                       INNER JOIN MANUALCIAM M ON M.CDFILIAL = P.CDFILIAL AND
                                                  M.CDCIA = P.CDCIA AND
                                                  M.CDMANCIA = P.CDMANCIA
                 INNER JOIN SUBGRUSEMAM S 
                    ON S.CDFILIAL =              P.CDFILIAL AND
                       S.CDCIA =                 P.CDCIA AND
                       S.CDMANCIA =              P.CDMANCIA AND
                       S.NRSERVICOCIAFILI =      P.NRSERVICOCIAFILI AND
                       S.CDCLASSE =              P.CDCLASSE AND
                       S.CDSUBGRU =              P.CDSUBGRU AND
                       S.NRSEQSUBGRUSEMAM =      P.NRSEQSUBGRUSEMAM
                 WHERE P.CDFILIAL =              :CDFILIAL
                   AND P.CDCIA =                 :CDCIA
                   AND P.CDMANCIA =              :CDMANCIA
                   AND P.CDCLASSE =              :CDCLASSE
                   AND P.CDCICLO =               :CDCICLO
                   AND P.NRSERVICOCIAFILIPRECO = :NRSERVICOCIAFILI
                   AND((S.DTINIVGSUMA BETWEEN :DTINIVG AND :DTFIMVG)
                        OR(S.DTFIMVGSUMA BETWEEN :DTINIVG AND :DTFIMVG)
                        OR(:DTFIMVG BETWEEN S.DTINIVGSUMA AND S.DTFIMVGSUMA)
                        OR(:DTINIVG BETWEEN S.DTINIVGSUMA AND S.DTFIMVGSUMA)
                        OR M.IDSITUAPROPCIA                            = 'P')
                 ORDER BY 1, 2";
		$stmt = $this->connection->prepare($sql);
		$stmt->execute(
			array(
				"CDFILIAL" =>         $cdfilial,
				"CDCIA" =>            $cdcia,
				"CDMANCIA" =>         $cdmancia,
				"CDCLASSE" =>         $cdclasse,
				"CDCICLO" =>          $cdciclo,
				"NRSERVICOCIAFILI" => $nrservicociafili,
				"DTINIVG" =>          $dtinivigse,
				"DTFIMVG" => 		  $dtfimvigse
			)
		);

		$subgrupos = $stmt->fetchAll(\PDO::FETCH_ASSOC);

		return $subgrupos;

	}
	
	private function same($arr) {
		return $arr === array_filter($arr, function ($element) use ($arr) {
			return ($element === $arr[0]);
		});
	}

	
	public function saveServ (DTO\Request\DataSet $request, DTO\Response $response){
		
		$this->dataSourceName = 'precoseciam';

		foreach ($request->getDataSet()->getRows() as $row) {
		
			$condition = array(
				array(
					"columnName" => "CDFILIAL",
					"operator" => "=",
					"value" => $row["CDFILIAL"]
				),
				array(
					"columnName" => "CDCIA",
					"operator" => "=",
					"value" => $row["CDCIA"]
				),
				array(
					"columnName" => "NRTABPRECO",
					"operator" => "=",
					"value" => $row["NRTABPRECO"]
				),
				array(
					"columnName" => "CDCLASSE",
					"operator" => "=",
					"value" => $row['CDCLASSE']
				),
				array(
					"columnName" => "CDCICLO",
					"operator" => "=",
					"value" => $row["CDCICLO"]
				),
				array(
					"columnName" => "NRSERVICOCIAFILI",
					"operator" => "=",
					"value" => $row["NRSERVICOCIAFILI"]
				)
			);
			$conditionTabPrecose = array(
				array(
					"columnName" => "CDFILIAL",
					"operator" => "=",
					"value" => $row["CDFILIAL"]
				),
				array(
					"columnName" => "CDCIA",
					"operator" => "=",
					"value" => $row["CDCIA"]
				),
				array(
					"columnName" => "NRTABPRECO",
					"operator" => "=",
					"value" => $row["NRTABPRECO"]
				)
			);

			$this->salvaTipoPreco($row);
			$proporcoes = array();

				if ($row["__is_new"]) {
					if($row['CDMANCIA'] !== null && $row['IDVINCULOMANUAL'] == 'S' && $row['IDSERVICO'] != "3" && $row['IDSERVICO'] != "5") {

		    			$proporcoes = $this->retornaProporcoes($row['CDFILIAL'], $row['CDCIA'], $row['CDMANCIA'], $row['CDCLASSE'], $row['CDCICLO'], $row['NRSERVICOCIAFILI'], $row['DTINIVIGSE'], $row['DTFIMVIGSE']);
						$countProporcoes = count($proporcoes);
						if($countProporcoes) {

							$idpropsubsemam = array();
							$qtpropsubsemam = array();

							$contador = 0;
							
							for($i=0;$i<count($proporcoes);$i++) {
								
								if($proporcoes[$i]['IDPROPSUBSEMAM'] == null){
									// YOU SHALL NOT PASS!!!
									$contador++;
									// return $response->setError(new DTO\Response\Error($this->messages->getMessage('Operação bloqueada. Item de faturamento vinculado a subgrupo sem proporção definida.'), 1));
								}else{
									if($proporcoes[$i]['QTPROPSUBSEMAM'] == null && $proporcoes[$i]['IDPROPSUBSEMAM'] != '1' && $proporcoes[$i]['IDPROPSUBSEMAM'] != '3'){
										// YOU SHALL NOT PASS!!!
										$contador++;
										// return $response->setError(new DTO\Response\Error($this->messages->getMessage('Operação bloqueada. Item de faturamento vinculado a subgrupo sem proporção definida.'), 1));
									}
									$idpropsubsemam[] = $proporcoes[$i]['IDPROPSUBSEMAM'];
									$qtpropsubsemam[] = $proporcoes[$i]['QTPROPSUBSEMAM'];
								}
							}		

							if($contador == 0) {
								/*
									 ---  11/08/2017  ---
									Controle de proporções agora é feita de acordo com a nova tabela de SUBGRUTABPRECOM
									cada proporção do subgrupo vai ser "virtual" e de acordo com a data do Subgrupo e a Data de Referência
									filtrada na tela.

								if($countProporcoes == 1) {

									$row['IDPROPORCAO'] = $idpropsubsemam[0];
	
								}elseif($countProporcoes > 1) {
	
									if(!$this->same($idpropsubsemam)) {
	
										$row['IDPROPORCAO'] = self::PROPORCAO_HUM_POR_PAX;
	
									}else{
	
										if($idpropsubsemam[0] == self::PROPORCAO_VARIAVEL) {
	
											if(!$this->same($qtpropsubsemam)) {
	
												$row['IDPROPORCAO'] = self::PROPORCAO_HUM_POR_PAX;
	
											}else{
	
												$row['IDPROPORCAO'] = self::PROPORCAO_VARIAVEL;
	
											}
											
										}else if($idpropsubsemam[0] == self::PROPORCAO_TABELA) {
	
											$row['IDPROPORCAO'] = self::PROPORCAO_HUM_POR_PAX;
	
										}else if($idpropsubsemam[0] == self::PROPORCAO_LOADING) {
	
											$row['IDPROPORCAO'] = self::PROPORCAO_HUM_POR_PAX;
	
										}else if($idpropsubsemam[0] == self::PROPORCAO_QUANTIDADE_FIXA) {
	
											$row['IDPROPORCAO'] = self::PROPORCAO_QUANTIDADE_FIXA;
	
										}
									}
	
								}  */
								$row['IDPROPORCAO'] = self::PROPORCAO_HUM_POR_PAX;
							}else{
								return $response->setError(new DTO\Response\Error('COC21000_IT_VINC_SUBGRUPO_SEM_PROP', 1));
							}
						}else{
							$row['IDPROPORCAO'] = self::PROPORCAO_HUM_POR_PAX;
							return $response->setError(new DTO\Response\Error('COC21000_IT_VINC_SUBGRUPO_N_VIGENTE', 1));
						}					
		    		} else {
		    			$row['SAVE_SUBGRUPO'] = false;
		    		}
				}else {
					// Flag para não incluir na tabela de SUBGRUTABPRECOM
					$row['SAVE_SUBGRUPO'] = false;
					if($row["VRPRESERV_BKP"] !== $row["VRPRESERV"] ||
						$row["VRPRESERVD_BKP"] !== $row["VRPRESERVD"]) {
		
						$cdfilial = $row['CDFILIAL'];
						$cdcia = $row['CDCIA'];
						$nrservicociafili = $row['NRSERVICOCIAFILI'];
						$cdclasse = $row['CDCLASSE'];
						$cdciclo = $row['CDCICLO'];
						$dtinivigse = $row['DTINIVIGSE'];
						$dtfimvigse = $row['DTFIMVIGSE'];
		
						if($this->verificaPlanoDeVoo($cdfilial, $cdcia, $nrservicociafili, $cdclasse, $cdciclo, $dtinivigse, $dtfimvigse)) {
							return $response->setError(new DTO\Response\Error("COC21000_IT_POSSUI_PLANOVOO", 1));
						}
					}
				}
			try {
				$this->connection->beginTransaction();

				$filterCriteriaTabprecose = new FilterCriteria($this->dataSourceNameTabprecose, $conditionTabPrecose);
		        $newFilterCriteriaTabprecose = $this->crudService->populateParameterBag($filterCriteriaTabprecose, $this->parameterBagColumns);
		        //Registros encontrados na tabela de preços Tabprecose
		        $dataSetTabela = $this->dataSourceManager->findBy($newFilterCriteriaTabprecose);
		        $Tabprecose = $dataSetTabela->getRows();
	
				$filterCriteria = new FilterCriteria($this->dataSourceName, $condition);
		        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		        //Registros encontrados na Serviços da Tabela Precoseciam
		        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
		        $registrosEncontrados = $dataSet->getRows();
	
		        //Cria novo request para cada Row antes de salvar
		        $dataSet = array();
					
	    		array_push($dataSet, $row);
	    		$dataSetServ = new DataSet($this->dataSourceNameTabprecose, $dataSet);
		    	$requestServ = new DTO\Request\DataSet($dataSetServ, $request->getMethod(), $request->getRoutePath(), $request->getUserId());

		        if(empty($registrosEncontrados)){
		        	if(($this->returnStrToTime($row['DTINIVIGSE']) >= $this->returnStrToTime($Tabprecose[0]['DTINIVIGTAB'])) && ($this->returnStrToTime($row['DTFIMVIGSE']) <= $this->returnStrToTime($Tabprecose[0]['DTFIMVIGTAB']))) {
						$this->effectSaveServ($proporcoes, $row, $requestServ, $response);
		        	}else{
		        		return $response->setError(new DTO\Response\Error('COC21000_IT_VIG_FORA', 1));
		        	}
		        }else{
		        	if($row['__is_new']){
		        		if(($this->returnStrToTime($row['DTINIVIGSE']) >= $this->returnStrToTime($Tabprecose[0]['DTINIVIGTAB'])) && ($this->returnStrToTime($row['DTFIMVIGSE']) <= $this->returnStrToTime($Tabprecose[0]['DTFIMVIGTAB']))){
			        		foreach ($registrosEncontrados as $registro){
				        		if(!($this->returnStrToTime($row['DTINIVIGSE']) > $this->returnStrToTime($registro["DTFIMVIGSE"]) ||
				        			 $this->returnStrToTime($row['DTFIMVIGSE']) < $this->returnStrToTime($registro["DTINIVIGSE"]))){
									return $response->setError(new DTO\Response\Error('COC21000_IT_JA_CAD_PERIODO', 1));
				        		}
				        	}
			        		$this->effectSaveServ($proporcoes, $row, $requestServ, $response);
			        	}else{
			        		return $response->setError(new DTO\Response\Error('COC21000_IT_VIG_FORA', 1));
			        	}
		        	}else{
		        		if(($this->returnStrToTime($row['DTINIVIGSE']) >= $this->returnStrToTime($Tabprecose[0]['DTINIVIGTAB'])) && ($this->returnStrToTime($row['DTFIMVIGSE']) <= $this->returnStrToTime($Tabprecose[0]['DTFIMVIGTAB']))){
			        		foreach ($registrosEncontrados as $registro){
			        			if((int)$row['NRSEQPRECO'] !== (int)$registro['NRSEQPRECO']){
				        			if(!($this->returnStrToTime($row['DTINIVIGSE']) > $this->returnStrToTime($registro["DTFIMVIGSE"]) || $this->returnStrToTime($row['DTFIMVIGSE']) < $this->returnStrToTime($registro["DTINIVIGSE"]))){
										return $response->setError(new DTO\Response\Error('COC21000_IT_JA_CAD_PERIODO', 1));
					        		}
			        			}
				        	}
				        	/*	-- 22/09/2017 -- */
				        	/* Atualização de vinculos do Subgrupo com o Item de Faturamento 
				        		não são feitas em casos em que houve apenas alterações
				        		no Item de Faturamento. */
			        		parent::save($requestServ, $response);
				        }else{
			        		return $response->setError(new DTO\Response\Error('COC21000_IT_VIG_FORA', 1));
				        }
		        	}
		        }
		        $this->connection->commit();
			} catch(\Exception $e) {
	        	$this->connection->rollback();
	        }
	    }
	}

	private function effectSaveServ($proporcoes, $row, $request, $response){
		$dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
		$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
		$this->crudService->save($dataSourceName, $rows);
        $nrseqpreco = $this->crudService->getColumnsToReturn()[0]['NRSEQPRECO'];
        if($row['SAVE_SUBGRUPO'] && !empty($proporcoes))
        	$this->saveSubgrupoTabelaPreco($nrseqpreco, $row, $request, $proporcoes);
        else
        	$response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
	}

	private function saveSubgrupoTabelaPreco($nrseqpreco, $row, $request, $proporcoes){
		$subGruposSave = array();
		if($nrseqpreco){
			foreach ($proporcoes as $proporcao) {
				array_push($subGruposSave, $this->createRowSubgrupo($nrseqpreco, $row, $request, $proporcao));
			}
			$this->effectSaveSubgrupo($subGruposSave, $request);
		}
	}

	private function effectSaveSubgrupo($dataSubgrupo, $response){
        $dataSet = new DataSet('subgrutabprecom', $dataSubgrupo);
        $request = new DTO\Request\DataSet($dataSet, '','','');
        $rowToSave = $this->crudService->getRowsToSave($request, 'subgrutabprecom');
        $this->crudService->save('subgrutabprecom', $rowToSave);
	}

	private function createRowSubgrupo($nrseqpreco, $row, $request, $proporcao){
		return array(
	        "CDFILIAL"				=> $row['CDFILIAL'],                 
	        "CDCIA" 				=> $row['CDCIA'], 
	        "NRTABPRECO"			=> $row['NRTABPRECO'], 
	        "NRSERVICOCIAFILIPRECO" => $proporcao['NRSERVICOCIAFILIPRECO'],
	        "CDCLASSE"				=> $row['CDCLASSE'], 
	        "CDCICLO"				=> $row['CDCICLO'],
	        "NRSEQPRECO"			=> intval($nrseqpreco),
	        "NRSUBGRUTABPRECOM" 	=> NULL,
	        "__is_new"				=> true,
	        "__is_new_code" 		=> true,
	        "__new_code_pk" 		=> "SUBGRUTABPRECOM".$row['CDFILIAL'].$row['CDCIA'].$row['NRTABPRECO'].intval($nrseqpreco),
	        "__table_pk"			=> "NRSUBGRUTABPRECOM",
	        "__new_code_length" 	=> 5,
	        "CDMANCIA"				=> $row['CDMANCIA'],
	        "NRSERVICOCIAFILI"		=> $proporcao['NRSERVICOCIAFILI'],
	        "CDSUBGRU"				=> $proporcao['CDSUBGRU'],
	        "NRSEQSUBGRUSEMAM"		=> $proporcao['NRSEQSUBGRUSEMAM'],
	        "CDPRODUTO" 			=> $proporcao['CDPRODUTO'],        
	        "CDRECEITA" 			=> $proporcao['CDRECEITA'],        
	        "IDPROPSUBSEMAM"		=> $proporcao['IDPROPSUBSEMAM'],       
	        "QTPROPSUBSEMAM"		=> $proporcao['QTPROPSUBSEMAM'],
			"DTFIMVGSUMAINCLUSAO"   => $proporcao['DTFIMVGSUMA'],
			"DTINIVGSUMAINCLUSAO"   => $proporcao['DTINIVGSUMA'],
	        "VRPRECOSUBGRU" 		=> 0.00,  
	        "VRPRECOSUBGRUD"		=> 0.00  
		);
	}
	
	public function salvaTipoPreco($row){
        $sql = "UPDATE SERVICOCIAFILI SET IDTPPRECO = :IDTPPRECO WHERE CDFILIAL = :CDFILIAL AND CDCIA = :CDCIA AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(array(
            'IDTPPRECO' => $row["IDTPPRECO"],
            'CDFILIAL' => $row["CDFILIAL"],
            'CDCIA' => $row["CDCIA"],
            'NRSERVICOCIAFILI' => $row["NRSERVICOCIAFILI"]
        ));
	}

	public function savePrecoPromo(DTO\Request\Row $request, DTO\Response $response){
		$row = $request->getRow();
		$this->dataSourceName = $this->dataSourceNamePrecoPromo;
		try {
			$servico =array(array(
				"CDFILIAL" => $row["CDFILIAL"],
				"CDCIA" => $row["CDCIA"],
				"NRTABPRECO" => $row["NRTABPRECO"],
				"CDCLASSE" => $row["CDCLASSE"],
				"CDCICLO" => $row["CDCICLO"],
				"NRSEQPRECO" => $row["NRSEQPRECO"],
				"NRSERVICOCIAFILI" => $row["NRSERVICOCIAFILI"]
			));
			$permiteSave=0;
			$precos = $this->getPrecoPromoforServ($servico, $row);
			if(count($precos)>0){
				for($key=0; $key<count($precos[0]); $key++){
					if($key==count($precos[0])-1){
						if($row['QTPAXINICIAL']>$precos[0][$key]['QTPAXFINAL']){
							$permiteSave++;
						}
						if($row['QTPAXFINAL'] < $precos[0][$key]['QTPAXINICIAL']){
							$permiteSave++;
						}
					}else {
						if($row['QTPAXINICIAL']<$precos[0][$key]['QTPAXINICIAL']
							&& $row['QTPAXFINAL']<$precos[0][$key]['QTPAXINICIAL']){
							$permiteSave++;
						}
						if($row['QTPAXINICIAL']>$precos[0][$key]['QTPAXFINAL']
							&& $row['QTPAXFINAL'] < $precos[0][$key+1]['QTPAXINICIAL']){
							$permiteSave++;
						} else if($row['QTPAXINICIAL']>$precos[0][$key]['QTPAXFINAL']
							&& $row['QTPAXFINAL'] > $precos[0][$key]['QTPAXINICIAL'])
							$permiteSave++;
					}
				}
			}
			if(sizeOf($precos) < 1)
				parent::save($request, $response);
		    else if($permiteSave >= count($precos[0]))
				parent::save($request, $response);
			else
				throw new \Exception("COC21000_QT_PASSAGEIROS_INVALIDA", 1);

		} catch(\Exception $e){
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	public function getNewCodePK($row){
		$castNumber = empty($row['__cast_number']) ? false : $row['__cast_number'];
        $newCodeKey = empty($row['__using_unit']) ? $row['__new_code_pk'] : ($row['__new_code_pk'].$this->environment->getCdFilial());
        $newCodeKey = strtoupper($newCodeKey);
        $row_pk[strtoupper($row['__table_pk'])] = $this->newCode->getCodeByTableName($newCodeKey, $row['NRORG'], 1, $row['__new_code_length'], $castNumber);
		return $row_pk[strtoupper($row['__table_pk'])];
	}

	public function returnColumns(){
    	return $this->crudService->getColumnsToReturn();
    }

	private function returnStrToTime($data){
		$data = explode(' ', $data)[0];
		$date = \DateTime::createFromFormat('d/m/Y', $data, new \DateTimeZone('America/Sao_Paulo'));
    	return $date->format('Y-m-d');
    }

	public function calculaPorcentagem(DTO\Request $request, DTO\Response $response){

        try {

            $row = $request->getDataSet()->getRows()[0];

			$manuais = array();

			for($i=0;$i<count($row["CDMANCIA"]);$i++) {
				$manuais[] = $row["CDMANCIA"][$i]["CDMANCIA"];
			}

			$row['CDMANCIA'] = implode("_", $manuais);

            $procedure = new StoredProcedure($this->connection, 'AUMENTA_PERCENTUAL_PRECO');

            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDCIA', Param::PARAM_INPUT, $row['CDCIA'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRTABPRECO', Param::PARAM_INPUT, $row['NRTABPRECO'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_GERATABELA', Param::PARAM_INPUT, $row['GERATABELA'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NOMETABELA', Param::PARAM_INPUT, $row['NMTABPRECO'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSERVICOCIAFILI', Param::PARAM_INPUT, $row['NRSERVICOCIAFILI'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPERCENTUAL', Param::PARAM_INPUT, $row['VRPERCENTUAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DATAINIVIGENCIA', Param::PARAM_INPUT, $row['DTINIVIGTAB'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DATAFIMVIGENCIA', Param::PARAM_INPUT, $row['DTFIMVIGTAB'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->crudService->getEnvironment()->getCdOperador(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $this->crudService->getEnvironment()->getNrOrg(), Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_CDMANCIA', Param::PARAM_INPUT, $row['CDMANCIA'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLISTAERROS', Param::PARAM_OUTPUT, null, null, 200));
            $result = $procedure->execute();

			if($result['P_NRLISTAERROS'] != null) {
				$this->MessagesProcedure->getErrorMessages($row['CDFILIAL'], $result['P_NRLISTAERROS'], $response, null);
            }

        } catch (\Exception $e){
            $response->setError(new Error($this->MessagesProcedure->cleanMessage($e->getMessage()), 500));
        }
	}

    private function getPrecoPromoforServ($servicos, $row){
	    $registros = array();
    	foreach($servicos as $servico) {
	    	$condition = array(
				array(
					"columnName" => "CDFILIAL",
					"operator" => "=",
					"value" => $servico["CDFILIAL"]
				),
				array(
					"columnName" => "CDCIA",
					"operator" => "=",
					"value" => $servico["CDCIA"]
				),
				array(
					"columnName" => "NRTABPRECO",
					"operator" => "=",
					"value" => $servico["NRTABPRECO"]
				),
				array(
					"columnName" => "CDCLASSE",
					"operator" => "=",
					"value" => $servico["CDCLASSE"]
				),
				array(
					"columnName" => "CDCICLO",
					"operator" => "=",
					"value" => $servico["CDCICLO"]
				),
				array(
					"columnName" => "NRSEQPRECO",
					"operator" => "=",
					"value" => $servico["NRSEQPRECO"]
				),
				array(
					"columnName" => "NRSERVICOCIAFILI",
					"operator" => "=",
					"value" => $servico["NRSERVICOCIAFILI"]
				)
			);
    		$filterCriteriaPrecoPromo = new FilterCriteria($this->dataSourceNamePrecoPromo, $condition);
	        $newFilterCriteriaPrecoPromo = $this->crudService->populateParameterBag($filterCriteriaPrecoPromo, $this->parameterBagColumns);
	        $dataSetPrecoPromo = $this->dataSourceManager->findBy($newFilterCriteriaPrecoPromo);
	        $registrosEncontrados = $dataSetPrecoPromo->getRows();
	        if(!empty($registrosEncontrados))
    			array_push($registros, $registrosEncontrados);
    	}
	    return $registros;
	}

    private function updatePrecoPromo($servicos, $row, $response){
    	//Procura Preços Promocionais dos Serviços e, caso encontre, updata seus valores
    	$registros = $this->getPrecoPromoforServ($servicos, $row);
	        if(!empty($registros)){
	        	//Encontrou Registros na tabela de Preço Promocional
	        	// foreach ($registros as &$array) {
	        	// 	//Retira uma camada do array criada ao pesquisar os Preços
	        	// 	$array = $array[0];
	        	// }
	        	// unset($array);
	    		$this->dataSourceName = "proprecosem";
	        	for($serv =0; $serv<count($registros); $serv++){
	        		for($promo=0; $promo<count($registros[$serv]); $promo++){
	        			$registros[$serv][$promo]['VRPRESERVPRO'] = (($row['PERCENT']/100) + 1 ) * $registros[$serv][$promo]['VRPRESERVPRO'];
						$registros[$serv][$promo]['VRPRESERVDPRO'] = (($row['PERCENT']/100) + 1 ) * $registros[$serv][$promo]['VRPRESERVDPRO'];
	        		}
	        		$newDataSet = new DataSet($this->dataSourceNamePrecoPromo, $registros[$serv]);
	    			$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
	    			//Chama o save do TeknisaCrud para salvar as alterações nos Preços Promocionais
	    			parent::save($newRequest, $response);
	        	}
	        }
    }

    private function alterPrecoPromo($precoPromo, $columnsSaveServ, $row, $response, $request){
    	//Retira a primeira posição do array, referente as coluns salvas na Tabela de Preço
    	array_shift($columnsSaveServ);
    	foreach ($precoPromo as &$preco) {
    		$preco[0]['VRPRESERVPRO'] = (($row['PERCENT']/100) + 1 ) * $preco[0]['VRPRESERVPRO'];
			$preco[0]['VRPRESERVDPRO'] = (($row['PERCENT']/100) + 1 ) * $preco[0]['VRPRESERVDPRO'];
			foreach ($columnsSaveServ as &$newServico) {
				if($preco[0]['NRSERVICOCIAFILI'] == $newServico['NRSERVICOCIAFILI']){
					$preco[0]['CDFILIAL'] = $newServico['CDFILIAL'];
					$preco[0]['CDCIA'] = $newServico['CDCIA'];
					$preco[0]['NRTABPRECO'] = $newServico['NRTABPRECO'];
					$preco[0]['NRSERVICOCIAFILI'] = $newServico['NRSERVICOCIAFILI'];
					$preco[0]['NRSEQPRECO'] = $newServico['NRSEQPRECO'];
					$preco[0]['__is_new'] = true;
				}
			}
			unset($newServico);
	   	}
	   	unset($preco);
	   	$arrayPromo = array();
	   	foreach ($precoPromo as $preco ) {
	   		array_push($arrayPromo, $preco[0]);
	   	}
	   	$this->dataSourceName = $this->dataSourceNamePrecoPromo;
	   	$dataSetPromo = new DataSet($this->dataSourceNamePrecoPromo, $arrayPromo);
		$requestPromo = new DTO\Request\DataSet($dataSetPromo, "POST", "/crud/proprecosem/save", $request->getUserId());
		parent::save($requestPromo, $response);
		$this->dataSourceName = 'precoseciam';
    }

     //Workflow proccess
	public function createWorkflow (DTO\Request\DataSet $request, DTO\Response $response){
		try {
			$row = $request->getDataSet()->getRows()[1];
			$dataToSave = $request->getDataSet()->getRows()[0];
			foreach ($dataToSave as $key => $modificacoes) {
				$primaryKeys = array(
					'CDCIA' => $row['CDCIA'],
					'CDFILIAL' => $row['CDFILIAL'],
					'NRORG' => $row['NRORG'],
					'NRTABPRECO' => $row['NRTABPRECO'],
					'NRSERVICOCIAFILI' => $row['NRSERVICOCIAFILI'],
					'NRSEQPRECO' => $row['NRSEQPRECO'],
					'CDCLASSE' => $row['CDCLASSE'],
					'CDCICLO' => $row['CDCICLO']
				);
				$nmitemwflow = $row['DSSERVICOCIAFILI'];
				$dsmotivowflow = $row['DSMOTIVOACAO'];
				$nmtabelawflow = 'PRECOSECIAM';
				$idtipoperwflow = $row['IDTIPOPERWFLOW'];
				$nritemprocaprova = $row['NRITEMPROCAPROVA'][$key];
				$nrprocessoaprova = $row['NRPROCESSOAPROVA'];
				$rowWorkflow = $this->workflowService->createRowToWorkFlowProcess((array)$modificacoes, $primaryKeys, $dsmotivowflow, $nmitemwflow, $nmtabelawflow, $idtipoperwflow, $nritemprocaprova, $nrprocessoaprova);
				$this->workflowService->createWorkFlowProcess($primaryKeys, (array)$modificacoes, $rowWorkflow/*, true, 'IDANALIZEATIV'*/);
			}
		} catch (\Exception $e){
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}
	private function existeItemFaturamento($row, $NRITEMPROCAPROVA, $NRPROCESSOAPROVA, $IDTIPOPERWFLOW, $IDSITWORKFLOW){
    	$existe = false;
    	$this->dataSourceName = 'quantidade_workflow';
			$condition = array(
				array(
					"columnName" => "NRITEMPROCAPROVA",
					"operator" => "=",
					"value" => '_'.$NRITEMPROCAPROVA.'_'
				),
				array(
					"columnName" => "NRORG",
					"operator" => "=",
					"value" => $row["NRORG"]
				),
				array(
					"columnName" => "NRPROCESSOAPROVA",
					"operator" => "=",
					"value" => $NRPROCESSOAPROVA
				),
				array(
					"columnName" => "IDTIPOPERWFLOW",
					"operator" => "=",
					"value" => '_'.$IDTIPOPERWFLOW.'_'
				),
				array(
					"columnName" => "IDSITWORKFLOW",
					"operator" => "=",
					"value" => $IDSITWORKFLOW
				),
				array(
					"columnName" => "DSPRIMARYKEY",
					"operator" => "=",
					"value" => 'CDCIA='.$row['CDCIA'].';'.'CDFILIAL='.$row['CDFILIAL'].';'.'NRORG='.$row['NRORG'].';'.'NRTABPRECO='.$row['NRTABPRECO'].';'.'NRSERVICOCIAFILI='.$row['NRSERVICOCIAFILI'].';'.'CDCLASSE='.$row['CDCLASSE'].';'.'CDCICLO='.$row['CDCICLO'].';'
				)
			);
		$this->parameterBagColumns = array('NRITEMPROCAPROVA', 'NRORG', 'NRPROCESSOAPROVA', 'IDTIPOPERWFLOW', 'IDSITWORKFLOW', 'DSPRIMARYKEY');
		$filterCriteriaPrecoPromo = new FilterCriteria($this->dataSourceName, $condition);
        $newFilterCriteriaPrecoPromo = $this->crudService->populateParameterBag($filterCriteriaPrecoPromo, $this->parameterBagColumns);
        $dataSetPrecoPromo = $this->dataSourceManager->findBy($newFilterCriteriaPrecoPromo);
        $registrosEncontrados = $dataSetPrecoPromo->getRows();
        if($registrosEncontrados[0]['QTD_WFLOW'] != 0)
			$existe = true;
	    return $existe;
	}

	public function createWorkflow_newItem (DTO\Request\DataSet $request, DTO\Response $response){
		try {
			$naoSalvou = 0;
			$dataToSave = $request->getDataSet()->getRows();
			$dsmotivowflow = $dataToSave[0]['DSMOTIVOACAO'];
			$idtipoperwflow = $dataToSave[0]['IDTIPOPERWFLOW'];
			$nritemprocaprova = $dataToSave[0]['NRITEMPROCAPROVA'];
			$nrprocessoaprova = $dataToSave[0]['NRPROCESSOAPROVA'];
			unset($dataToSave[0]['DSMOTIVOACAO']);
			unset($dataToSave[0]['IDTIPOPERWFLOW']);
			unset($dataToSave[0]['NRITEMPROCAPROVA']);
			unset($dataToSave[0]['NRPROCESSOAPROVA']);

			foreach ($dataToSave as $modificacoes) {
				if(!$this->existeItemFaturamento($modificacoes, $nritemprocaprova,
					$nrprocessoaprova, $idtipoperwflow, '2')){
					$primaryKeys = array(
						'CDCIA' => $modificacoes['CDCIA'],
						'CDFILIAL' => $modificacoes['CDFILIAL'],
						'NRORG' => $modificacoes['NRORG'],
						'NRTABPRECO' => $modificacoes['NRTABPRECO'],
						'NRSERVICOCIAFILI' => $modificacoes['NRSERVICOCIAFILI'],
						'CDCLASSE' => $modificacoes['CDCLASSE'],
						'CDCICLO' => $modificacoes['CDCICLO']
					);
					$modificacoes['NRSEQPRECO'] = $this->getNewCodePK($modificacoes);
					$nmitemwflow = $modificacoes['DSSERVICOCIAFILI'];
					$nmtabelawflow = 'PRECOSECIAM';
					if(isset($modificacoes['__new_code_pk']))
						unset($modificacoes['__new_code_pk']);
					if(isset($modificacoes['__is_new_code']))
						unset($modificacoes['__is_new_code']);
					if(isset($modificacoes['__table_pk']))
						unset($modificacoes['__table_pk']);
					if(isset($modificacoes['__new_code_length']))
						unset($modificacoes['__new_code_length']);
					if(isset($modificacoes['DSSERVICOCIAFILI']))
						unset($modificacoes['DSSERVICOCIAFILI']);
					$rowWorkflow = $this->workflowService->createRowToWorkFlowProcess((array)$modificacoes, $primaryKeys, $dsmotivowflow, $nmitemwflow, $nmtabelawflow, $idtipoperwflow, $nritemprocaprova, $nrprocessoaprova);
					$this->workflowService->createWorkFlowProcess($primaryKeys, (array)$modificacoes, $rowWorkflow/*, true, 'IDANALIZEATIV'*/);
				} else $naoSalvou++;
			}
			if($naoSalvou == count($dataToSave))
				$message = "COC21000_SOL_INC_BLOQ_IT_JA_CAD";
			else
				$message = "COC21000_SOL_INC_IT_GERADA";
			$response->addMessage(new DTO\Response\Message ($message));
		} catch (\Exception $e){
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}
	public function createWorkflow_deleteItem (DTO\Request\DataSet $request, DTO\Response $response){
		try {
			$dataToSave = $request->getDataSet()->getRows();
			foreach ($dataToSave as $key => $modificacoes) {
				$primaryKeys = array(
					'CDCIA' => $modificacoes['CDCIA'],
					'CDFILIAL' => $modificacoes['CDFILIAL'],
					'NRORG' => $modificacoes['NRORG'],
					'NRTABPRECO' => $modificacoes['NRTABPRECO'],
					'NRSERVICOCIAFILI' => $modificacoes['NRSERVICOCIAFILI'],
					'NRSEQPRECO' => $modificacoes['NRSEQPRECO'],
					'CDCLASSE' => $modificacoes['CDCLASSE'],
					'CDCICLO' => $modificacoes['CDCICLO']
				);
				$nmitemwflow = $modificacoes['DSSERVICOCIAFILI'];
				$dsmotivowflow = $modificacoes['DSMOTIVOACAO'];
				$nmtabelawflow = 'PRECOSECIAM';
				$idtipoperwflow = $modificacoes['IDTIPOPERWFLOW'];
				$nritemprocaprova = $modificacoes['NRITEMPROCAPROVA'];
				$nrprocessoaprova = $modificacoes['NRPROCESSOAPROVA'];
				//Retira as informações desnecessarias da Row
				unset($modificacoes['DSMOTIVOACAO']);
				unset($modificacoes['IDTIPOPERWFLOW']);
				unset($modificacoes['IDSITWORKFLOW']);
				unset($modificacoes['DSPRIMARYKEY']);
				unset($modificacoes['NMCAMPOWFLOW']);
				unset($modificacoes['NRITEMPROCAPROVA']);
				unset($modificacoes['NRPROCESSOAPROVA']);
				unset($modificacoes['DSSERVICOCIAFILI']);
				unset($modificacoes['IDSERVICO']);
				unset($modificacoes['SGCLASSE']);
				unset($modificacoes['IDCICLO']);
				unset($modificacoes['IDTIPMANUAL']);
				if(isset($modificacoes['__isSelected']))
					unset($modificacoes['__isSelected']);
				if(isset($modificacoes['__notSelectable']))
					unset($modificacoes['__notSelectable']);
				if(isset($modificacoes['__index']))
					unset($modificacoes['__index']);
				if(isset($modificacoes['__is_new']))
					unset($modificacoes['__is_new']);
				$rowWorkflow = $this->workflowService->createRowToWorkFlowProcess((array)$modificacoes, $primaryKeys, $dsmotivowflow, $nmitemwflow, $nmtabelawflow, $idtipoperwflow, $nritemprocaprova, $nrprocessoaprova);
				$this->workflowService->createWorkFlowProcess($primaryKeys, (array)$modificacoes, $rowWorkflow/*, true, 'IDANALIZEATIV'*/);
			}
		} catch (\Exception $e){
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}
	public function createWorkflow_precoPromo (DTO\Request\DataSet $request, DTO\Response $response){
		try {
			$dataToSave = $request->getDataSet()->getRows();
			for($key = 0; $key<(count($dataToSave)-1); $key++){
				$primaryKeys = array(
					'CDCIA' => $dataToSave[$key]['CDCIA'],
					'CDFILIAL' => $dataToSave[$key]['CDFILIAL'],
					'NRORG' => $dataToSave[$key]['NRORG'],
					'NRTABPRECO' => $dataToSave[$key]['NRTABPRECO'],
					'NRSERVICOCIAFILI' => $dataToSave[$key]['NRSERVICOCIAFILI'],
					'NRSEQPRECO' => $dataToSave[$key]['NRSEQPRECO'],
					'CDCLASSE' => $dataToSave[$key]['CDCLASSE'],
					'CDCICLO' => $dataToSave[$key]['CDCICLO'],
					'QTPAXFINAL' => $dataToSave[$key]['QTPAXFINAL'],
					'QTPAXINICIAL' => $dataToSave[$key]['QTPAXINICIAL']
				);
				$nmitemwflow = $dataToSave[count($dataToSave)-1]['NMTABPRECO'];
				$dsmotivowflow = $dataToSave[count($dataToSave)-1]['DSMOTIVOACAO'];
				$nmtabelawflow = 'PROPRECOSEM';
				$idtipoperwflow = $dataToSave[count($dataToSave)-1]['IDTIPOPERWFLOW'];
				$nritemprocaprova = $dataToSave[count($dataToSave)-1]['NRITEMPROCAPROVA'];
				$nrprocessoaprova = $dataToSave[count($dataToSave)-1]['NRPROCESSOAPROVA'];
                unset($dataToSave[$key]['CDCIA']);
                unset($dataToSave[$key]['CDCICLO']);
                unset($dataToSave[$key]['CDCLASSE']);
                unset($dataToSave[$key]['CDFILIAL']);
                unset($dataToSave[$key]['NRORG']);
                unset($dataToSave[$key]['NRSEQPRECO']);
                unset($dataToSave[$key]['NRSERVICOCIAFILI']);
                unset($dataToSave[$key]['NRTABPRECO']);
                unset($dataToSave[$key]['QTPAXFINAL']);
                unset($dataToSave[$key]['QTPAXINICIAL']);
                unset($dataToSave[$key]['__is_new']);
				$rowWorkflow = $this->workflowService->createRowToWorkFlowProcess((array)$dataToSave[$key], $primaryKeys, $dsmotivowflow, $nmitemwflow, $nmtabelawflow, $idtipoperwflow, $nritemprocaprova, $nrprocessoaprova);
				$this->workflowService->createWorkFlowProcess($primaryKeys, (array)$dataToSave[$key], $rowWorkflow/*, true, 'IDANALIZEATIV'*/);
			}
		} catch (\Exception $e){
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}

	private function excluirItemDeFaturamento($nrseqpreco, $cdciclo, $cdclasse, $nrservicociafili, $nrtabpreco, $cdcia, $cdfilial) {

		$sql = "DELETE FROM PRECOSECIAM WHERE CDCIA = :CDCIA AND CDFILIAL = :CDFILIAL AND NRSEQPRECO = :NRSEQPRECO AND CDCICLO = :CDCICLO AND CDCLASSE = :CDCLASSE AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI AND NRTABPRECO = :NRTABPRECO";

		$stmt = $this->connection->prepare($sql);
		$stmt->execute(array(
			"CDCIA" => $cdcia,
			"CDFILIAL" => $cdfilial,
			"NRSEQPRECO" => $nrseqpreco,
			"CDCICLO" => $cdciclo,
			"CDCLASSE" => $cdclasse,
			"NRSERVICOCIAFILI" => $nrservicociafili,
			"NRTABPRECO" => $nrtabpreco
		));

	}

	private function excluirSubgrupoTabelaPreco($nrseqpreco, $cdciclo, $cdclasse, $nrservicociafili, $nrtabpreco, $cdcia, $cdfilial, $cdmancia) {

		$sql = "DELETE FROM SUBGRUTABPRECOM WHERE CDCIA = :CDCIA AND CDFILIAL = :CDFILIAL AND NRSEQPRECO = :NRSEQPRECO AND CDCICLO = :CDCICLO AND CDCLASSE = :CDCLASSE AND NRSERVICOCIAFILIPRECO = :NRSERVICOCIAFILIPRECO AND NRTABPRECO = :NRTABPRECO AND CDMANCIA = :CDMANCIA";

		$stmt = $this->connection->prepare($sql);
		$stmt->execute(array(
			"CDCIA" => $cdcia,
			"CDFILIAL" => $cdfilial,
			"NRSEQPRECO" => $nrseqpreco,
			"CDCICLO" => $cdciclo,
			"CDCLASSE" => $cdclasse,
			"NRSERVICOCIAFILIPRECO" => $nrservicociafili,
			"NRTABPRECO" => $nrtabpreco,
			"CDMANCIA" => $cdmancia
		));

	}
	
	private function excluirPrecoProporcional($nrseqpreco, $cdciclo, $cdclasse, $nrservicociafili, $nrtabpreco, $cdcia, $cdfilial) {

		$sqlDeleteProprecosem = "DELETE FROM PROPRECOSEM WHERE CDCIA = :CDCIA AND CDFILIAL = :CDFILIAL AND NRSEQPRECO = :NRSEQPRECO AND CDCICLO = :CDCICLO AND CDCLASSE = :CDCLASSE AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI AND NRTABPRECO = :NRTABPRECO";

		$stmtProprecosem = $this->connection->prepare($sqlDeleteProprecosem);
		$stmtProprecosem->execute(array(
			"CDCIA" => $cdcia,
			"CDFILIAL" => $cdfilial,
			"NRSEQPRECO" => $nrseqpreco,
			"CDCICLO" => $cdciclo,
			"CDCLASSE" => $cdclasse,
			"NRSERVICOCIAFILI" => $nrservicociafili,
			"NRTABPRECO" => $nrtabpreco
		));

	}
	
	public function verificaFilhos(DTO\Request\DataSet $request, DTO\Response $response){
		
		$dataToSave = $request->getDataSet()->getRows();
		
		$contadorItensFatComPProp = 0;
		
		foreach ($dataToSave as $row) {
			
			if($this->possuiPrecoProporcionalItenFat($row)){
				$contadorItensFatComPProp++;
			}
			
		}
		
		$filhosEncontrados = array();
		
		array_push($filhosEncontrados, $contadorItensFatComPProp);
		$response->addDataSet(new DataSet("registroscompreprop", $filhosEncontrados));
	
	}
	
	private function possuiPrecoProporcionalItenFat($row){
		$filtro = array(
            array(
			    "columnName"=> "CDCIA",
			    "operator"=> "=",
			    "value"=> $row["CDCIA"]
            ),
            array(
			    "columnName"=> "CDFILIAL",
			    "operator"=> "=",
			    "value"=> $row["CDFILIAL"]
            ),
            array(
			    "columnName"=> "NRSEQPRECO",
			    "operator"=> "=",
			    "value"=> $row["NRSEQPRECO"]
            ),
            array(
			    "columnName"=> "CDCICLO",
			    "operator"=> "=",
			    "value"=> $row["CDCICLO"]
            ),
            array(
			    "columnName"=> "CDCLASSE",
			    "operator"=> "=",
			    "value"=> $row["CDCLASSE"]
            ),
            array(
			    "columnName"=> "NRSERVICOCIAFILI",
			    "operator"=> "=",
			    "value"=> $row["NRSERVICOCIAFILI"]
            ),
            array(
			    "columnName"=> "NRTABPRECO",
			    "operator"=> "=",
			    "value"=> $row["NRTABPRECO"] 
            )
		);
		
		$filterCriteriaPrecoProporcional = new FilterCriteria('proprecosem', $filtro);
		$newFilterCriteriaPrecoProporcional = $this->crudService->populateParameterBag($filterCriteriaPrecoProporcional, array());
		$dataSetPrecoProporcional = $this->dataSourceManager->findBy($newFilterCriteriaPrecoProporcional);
		
		$rows = $dataSetPrecoProporcional->getRows();
		 
		return (sizeOf($rows) > 0);
		
	}

	private function verificaPlanoDeVoo($cdfilial, $cdcia, $nrservicociafili, $cdclasse, $cdciclo, $dtinivigse, $dtfimvigse) {

		$sql = "SELECT COUNT(1) AS ITENS FROM SERVPLANOM S INNER JOIN PLANOVOOM P ON P.CDFILIAL = S.CDFILIAL AND P.CDCIA = S.CDCIA AND P.NRVOO = S.NRVOO AND P.CDTRECHO = S.CDTRECHO AND P.DTHRPLAVOOPV = S.DTHRPLAVOOPV WHERE S.CDFILIAL = :CDFILIAL AND S.CDCIA = :CDCIA AND S.NRSERVICOCIAFILI = :NRSERVICOCIAFILI AND S.CDCLASSE = :CDCLASSE AND S.CDCICLO = :CDCICLO AND P.CDFILIAL = :CDFILIAL AND P.CDCIA = :CDCIA AND P.DTHRABASTPV BETWEEN :DTINIVIGSE AND :DTFIMVIGSE";
   
		$stmt = $this->connection->prepare($sql);

		$stmt->execute(array(
			"CDFILIAL" => $cdfilial,
			"CDCIA" => $cdcia,
			"NRSERVICOCIAFILI" => $nrservicociafili,
			"CDCLASSE" => $cdclasse,
			"CDCICLO" => $cdciclo,
			"DTINIVIGSE" => $dtinivigse,
			"DTFIMVIGSE" => $dtfimvigse
		));

		$result = $stmt->fetch(\PDO::FETCH_ASSOC);
		if(isset($result['ITENS'])) {
			if($result['ITENS'] != self::NAO_POSSUI_PLANO_DE_VOO) {
				return true;
			}else{
				return false;
			}
		}
		return false;
	}
	
	private function buscaMenuGrid($row){
		$sql = "SELECT
					T.NRORG,
					T.CDFILIAL,
					T.CDCIA,
					T.NRTABPRECO,
					T.CDMANCIA,
					M.DSMANUAL,
					M.DTINIVGMA,
					M.DTFIMVGMA
				FROM
					TABPRECOMENUGRID T
					INNER JOIN MANUALCIAM M 
						ON  M.CDFILIAL = T.CDFILIAL
						AND M.CDCIA	= T.CDCIA
						AND M.CDMANCIA = T.CDMANCIA
				WHERE
						T.CDFILIAL	 = :CDFILIAL
					AND T.CDCIA 	 = :CDCIA
					AND T.NRTABPRECO = :NRTABPRECO
				ORDER BY
					DTINIVGMA,
					DTFIMVGMA DESC";
		$stmt = $this->connection->prepare($sql);
		$stmt->execute(array(
			"CDFILIAL" => $row['CDFILIAL'],
			"CDCIA" => $row['CDCIA'],
			"NRTABPRECO" => $row['NRTABPRECO']
		));
		return $stmt->fetchAll(\PDO::FETCH_ASSOC);
	}
	
	private function updataManualTabelaPreco($row){
	
        $sql = "UPDATE PRECOSECIAM SET CDMANCIA = :CDMANCIA WHERE NRSEQPRECO = :NRSEQPRECO AND CDCICLO = :CDCICLO AND CDCLASSE = :CDCLASSE AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI AND NRTABPRECO = :NRTABPRECO AND CDCIA = :CDCIA AND CDFILIAL = :CDFILIAL";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(array(
			"CDCIA" 			=> $row['CDCIA'],
			"CDCICLO"			=> $row['CDCICLO'],
            "CDMANCIA"			=> $row['CDMANCIA'],
			"CDCLASSE"			=> $row['CDCLASSE'],
			"CDFILIAL"			=> $row['CDFILIAL'],
			"NRTABPRECO"		=> $row['NRTABPRECO'],
			"NRSEQPRECO"		=> $row['NRSEQPRECO'],
			"NRSERVICOCIAFILI"  => $row['NRSERVICOCIAFILI']
        ));
	}
	
	public function refreshSubgrupos(DTO\Request\Row $request, DTO\Response $response){
		try {
			$this->connection->beginTransaction();
			$row = $request->getRow();
			$manuais = $this->buscaMenuGrid($row);
			if(!empty($manuais)){
				forEach($manuais as $manual) {
					$manual = $manual['CDMANCIA'];
					$proporcoes = $this->retornaProporcoes($row['CDFILIAL'], $row['CDCIA'], $manual, $row['CDCLASSE'], $row['CDCICLO'], $row['NRSERVICOCIAFILI'], $row['DTINIVIGSE'], $row['DTFIMVIGSE']);
					$this->importarItensService->deleteSubgrupoTabelaPreco($row['CDFILIAL'], $row['CDCIA'], $row['NRTABPRECO'], $row['NRSERVICOCIAFILI'], $row['CDCLASSE'], $row['CDCICLO'], $row['NRSEQPRECO'], $row['NRORG']);
					if(!empty($proporcoes) && ($row['IDSERVICO'] != "3" && $row['IDSERVICO'] != "5")){
						$row['CDMANCIA'] = $manual;
						$this->updataManualTabelaPreco($row);
						$this->saveSubgrupoTabelaPreco($row['NRSEQPRECO'], $row, $request, $proporcoes);
					}
				}
			}
			$this->connection->commit();
		} catch(\Exception $e){
        	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}

	public function atualizaVinculoSubgrupo(DTO\Request\Row $request, DTO\Response $response){
		try {
			$this->connection->beginTransaction();
			$row = $request->getRow();
			$manual = $this->validaVinculoMenuGrid($row);
			$proporcoes = $this->retornaProporcoes($row['CDFILIAL'], $row['CDCIA'], $manual['MAX(CDMANCIA)'], $row['CDCLASSE'], $row['CDCICLO'], $row['NRSERVICOCIAFILI'], $row['DTINIVIGSE'], $row['DTFIMVIGSE']);
			$this->importarItensService->deleteSubgrupoTabelaPreco($row['CDFILIAL'], $row['CDCIA'], $row['NRTABPRECO'], $row['NRSERVICOCIAFILI'], $row['CDCLASSE'], $row['CDCICLO'], $row['NRSEQPRECO'], $row['NRORG']);
			if(!empty($proporcoes) && ($row['IDSERVICO'] != "3" && $row['IDSERVICO'] != "5")){
				$row['CDMANCIA'] = $manual['MAX(CDMANCIA)'];
				$this->saveSubgrupoTabelaPreco($row['NRSEQPRECO'], $row, $request, $proporcoes);
			}
			$this->connection->commit();
		} catch(\Exception $e){
        	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}

	public function deleteItemFaturamento (DTO\Request\DataSet $request, DTO\Response $response) {

		try {

			$rows = $request->getDataSet()->getRows();

			$planoDeVoo = [];

			for($i=0; $i < count($rows); $i++) {

				$cdfilial = $rows[$i]['CDFILIAL'];
				$cdcia = $rows[$i]['CDCIA'];
				$nrservicociafili = $rows[$i]['NRSERVICOCIAFILI'];
				$cdclasse = $rows[$i]['CDCLASSE'];
				$cdciclo = $rows[$i]['CDCICLO'];
				$dtinivigse = $rows[$i]['DTINIVIGSE'];
				$dtfimvigse = $rows[$i]['DTFIMVIGSE'];
				$cdmancia = $rows[$i]['CDMANCIA'];

				if($this->verificaPlanoDeVoo($cdfilial, $cdcia, $nrservicociafili, $cdclasse, $cdciclo, $dtinivigse, $dtfimvigse)) {
					$planoDeVoo['POSSUI_PLANO_DE_VOO'][] = $rows[$i];
				}else{
					$planoDeVoo['NAO_POSSUI_PLANO_DE_VOO'][] = $rows[$i];
				}

			}
	
			if(isset($planoDeVoo['NAO_POSSUI_PLANO_DE_VOO'])) {

				for($j=0;$j < count($planoDeVoo['NAO_POSSUI_PLANO_DE_VOO']); $j++) {
					
					$nrseqpreco = $rows[$j]['NRSEQPRECO'];
					$cdciclo = $rows[$j]['CDCICLO'];
					$cdclasse = $rows[$j]['CDCLASSE'];
					$nrservicociafili = $rows[$j]['NRSERVICOCIAFILI'];
					$nrtabpreco = $rows[$j]['NRTABPRECO'];
					$cdcia = $rows[$j]['CDCIA'];
					$cdfilial = $rows[$j]['CDFILIAL'];
					if($this->possuiPrecoProporcionalItenFat($rows[$j])){
						$this->excluirPrecoProporcional($nrseqpreco, $cdciclo, $cdclasse, $nrservicociafili, $nrtabpreco, $cdcia, $cdfilial);
					}
					/*		 --- 09/08/2017 ---
						Tabela nova criada para vincular cada subgrupo adicionado com seu preço aberto
					*/
					$this->excluirSubgrupoTabelaPreco($nrseqpreco, $cdciclo, $cdclasse, $nrservicociafili, $nrtabpreco, $cdcia, $cdfilial, $cdmancia);
					
					$this->excluirItemDeFaturamento($nrseqpreco, $cdciclo, $cdclasse, $nrservicociafili, $nrtabpreco, $cdcia, $cdfilial);

				}

			}
	
		}catch(\Exception $e) {
			return $response->setError(new Error($e->getMessage(), 500));
		}
		
		if(isset($planoDeVoo['POSSUI_PLANO_DE_VOO'])) {

			$itensDeFaturamento = array();

			for($i=0;$i<count($planoDeVoo['POSSUI_PLANO_DE_VOO']);$i++) {

				$item = $planoDeVoo['POSSUI_PLANO_DE_VOO'][$i];
				
				$msg = ["DSSERVICOCIAFILI" => $item['DSSERVICOCIAFILI'], "SGCLASSE" => $item['SGCLASSE'], "IDCICLO" => $item['IDCICLO']];

				array_push($itensDeFaturamento, $msg);

			}

			$response->setError(new Error($itensDeFaturamento,1));

		}

	}
	
	private function validaVinculoMenuGrid($row){
		$manuais = $this->retornaQuantidadeManuais($row['CDFILIAL'], $row['CDCIA'], $row['NRTABPRECO'], $row['NRSERVICOCIAFILI'], $row['CDCLASSE'], $row['CDCICLO']);
		if(!empty($manuais)){
			if($manuais['COUNT(1)'] > 1){
				//Item de Faturamento vinculado à mais de um Menu Grid
				$msg = "COC21000_MAIS_DE_UM_VINCULO" . "#_#" . $row['DSSERVICOCIAFILI'] . "#_#" . $row['SGCLASSE'] . "#_#" . $row['IDCICLO'];
				throw new \Exception($msg, 500);
			} else {
				return $manuais;
			}
		} else {
			$msg = "COC21000_IT_SEM_VINCULO" . "#_#" . $row['DSSERVICOCIAFILI'] . "#_#" . $row['SGCLASSE'] . "#_#" . $row['IDCICLO'];
			//Nenhum manual vinculado encontrado
			throw new \Exception($msg, 500);
		}
	}

	public function deleteProprecosem (DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();

		$cdfilial = $row['CDFILIAL'];
        $cdcia = $row['CDCIA'];
        $cdciclo = $row['CDCICLO'];
        $cdclasse = $row['CDCLASSE'];
        $nrtabpreco = $row['NRTABPRECO'];
        $nrseqpreco = $row['NRSEQPRECO'];
        $nrservicociafili = $row['NRSERVICOCIAFILI'];
        $nrorg = $row['NRORG'];

        $this->importarItensService->deleteProprecosem($cdfilial, $cdcia, $nrtabpreco, $nrseqpreco, $cdclasse, $cdciclo, $nrservicociafili, $nrorg);
	}

	public function deleteProporcaopreco(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();

		$cdfilial = $row['CDFILIAL'];
        $cdcia = $row['CDCIA'];
        $cdciclo = $row['CDCICLO'];
        $cdclasse = $row['CDCLASSE'];
        $nrtabpreco = $row['NRTABPRECO'];
        $nrseqpreco = $row['NRSEQPRECO'];
        $nrservicociafili = $row['NRSERVICOCIAFILI'];
        $nrorg = $row['NRORG'];

        $this->importarItensService->deleteProporcaopreco($cdfilial, $cdcia, $nrtabpreco, $nrseqpreco, $cdclasse, $cdciclo, $nrservicociafili, $nrorg);
	}

	private function getVinculoSubgrupo($row) {
        $conditions = array(
            array (
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array (
                "columnName" => "CDCIA",
                "operator" => "=",
                "value" => $row['CDCIA']
            ),
            array (
                "columnName" => "NRSERVICOCIAFILI",
                "operator" => "=",
                "value" => $row['NRSERVICOCIAFILI']
            ),
            array (
                "columnName" => "CDCLASSE",
                "operator" => "=",
                "value" => $row['CDCLASSE']
            ),
            array (
                "columnName" => "CDCICLO",
                "operator" => "=",
                "value" => $row['CDCICLO']
            ),
            array (
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            ),
        );
        
        $dataSet = $this->crudService->getFilteredDataSet('planodevoo_between_vigencia', $conditions);
        
        if(empty($dataSet->getRows()))
            return [];
        else 
            return $dataSet->getRows();
    }

}